
define(['../../../views/requests', '../../../collections/requests', '../../../models/request', 'core_models/user'],
  function(RequestsView, RequestsCollection, RequestModel, UserModel) {

  describe('Requests view', function() {
    var view,
        collection,
        user;

    beforeEach(function() {

      user = new UserModel({
        role: 'admin'
      });

      collection = new RequestsCollection;

      sinon.stub(collection, 'fetch');

      view = new RequestsView({
        collection: collection
      });

      view.core.user = user;
    });

    it('displays a message when there are no requests', function() {
      view.render();

      expect(view.el.innerHTML).toMatch(/no certificates/);
    });

    xit('displays a message when unable to fetch requests', function() {
      // deferred until error reporting is done
    });

    it('clears requests via the collection', function() {
      var mock = sinon.mock(collection).expects('clear_requests').once();

      view.clear_requests();

      mock.verify();
    });

    describe('UI elements', function() {
      describe('open requests count', function() {
        it('shows when there are no open requests', function() {
          view.render();

          expect(view.ui.open_count.text()).toMatch(/Pending node requests: 0/);
        });

        it('shows when there is 1 open request', function() {
          sinon.stub(collection, 'open_requests').returns({length: 1});

          view.render();

          expect(view.ui.open_count.text()).toMatch(/Pending node requests: 1/);
        });

        it('shows when there is more than 1 open request', function() {
          sinon.stub(collection, 'open_requests').returns({length: 42});

          view.render();

          expect(view.ui.open_count.text()).toMatch(/Pending node requests: 42/);
        });

        it('updates when the collection count changes, without rerendering', function() {

          var stub = sinon.stub(collection, 'open_requests').returns({length: 1});

          view.render();

          stub.returns({length: 42});

          collection.trigger('change:status');

          expect(view.ui.open_count.text()).toMatch(/Pending node requests: 42/);

        });
      });

      describe('clear accepted/rejected requests', function() {
        it('shows when there are no completed requests', function() {
          view.render();

          expect(view.ui.clear_requests.text()).toMatch('no accepted/rejected');
          expect(view.ui.clear_requests.attr('disabled')).toBe('disabled');
        });

        it('shows when there is 1 completed request', function() {
          sinon.stub(collection, 'completed_requests').returns({length: 1});

          view.render();

          expect(view.ui.clear_requests.text()).toMatch('1 accepted/rejected');
          expect(view.ui.clear_requests.attr('disabled')).toBeFalsy();
        });

        it('shows when there is more than 1 completed request', function() {
          sinon.stub(collection, 'completed_requests').returns({length: 42});

          view.render();

          expect(view.ui.clear_requests.text()).toMatch('42 accepted/rejected');
          expect(view.ui.clear_requests.attr('disabled')).toBeFalsy();
        });

        it('updates when a model changes, without rerendering', function() {

          var stub = sinon.stub(collection, 'completed_requests').returns({length: 1});

          view.render();

          stub.returns({length: 42});

          collection.trigger('change:status');

          expect(view.ui.clear_requests.text()).toMatch('42 accepted/rejected');

        });

        it('updates when the collection count changes, without rerendering', function() {
          var stub = sinon.stub(collection, 'completed_requests').returns({length: 1});

          view.render();

          stub.returns({length: 42});

          // Marionette expects the remove event to include a model - we
          // send an object with an ID instead to fake it
          collection.trigger('remove', {id: null});

          expect(view.ui.clear_requests.text()).toMatch('42 accepted/rejected');

        });
      });

      describe('accept/reject all buttons', function() {
        it('are disabled when there are no actionable requests', function() {
          var stub = sinon.stub(collection, 'actionable_requests').returns({length: 0});

          view.render();

          expect(view.ui.accept_all.attr('disabled')).toBe('disabled');
          expect(view.ui.reject_all.attr('disabled')).toBe('disabled');
        });

        it('are enabled when there are actionable requests', function() {
          var stub = sinon.stub(collection, 'actionable_requests').returns({length: 1});

          view.render();

          expect(view.ui.accept_all.attr('disabled')).toBeFalsy();
          expect(view.ui.reject_all.attr('disabled')).toBeFalsy();
        });

        it('updates when the actionable count changes, without rerendering', function() {
          var stub = sinon.stub(collection, 'actionable_requests').returns({length: 1});

          view.render();

          stub.returns({length: 0});

          collection.trigger('change:status');

          expect(view.ui.accept_all.attr('disabled')).toBe('disabled');
          expect(view.ui.reject_all.attr('disabled')).toBe('disabled');
        });

        it('are disabled after accepting all open requests', function() {
          var deferred = $.Deferred().resolve();

          sinon.stub(collection, 'accept_all').returns(deferred);

          var stub = sinon.stub(collection, 'actionable_requests').returns({length: 1});

          view.render();

          stub.returns({length: 0});

          view.accept_all();

          expect(view.ui.accept_all.attr('disabled')).toBe('disabled');
          expect(view.ui.reject_all.attr('disabled')).toBe('disabled');
        });

        it('are disabled after rejecting all open requests', function() {
          var deferred = $.Deferred().resolve();

          sinon.stub(collection, 'reject_all').returns(deferred);

          var stub = sinon.stub(collection, 'actionable_requests').returns({length: 1});

          view.render();

          stub.returns({length: 0});

          view.reject_all();

          expect(view.ui.accept_all.attr('disabled')).toBe('disabled');
          expect(view.ui.reject_all.attr('disabled')).toBe('disabled');
        });
      });
    });

    describe('actioning requests', function() {

      var deferred,
          child_view;

      beforeEach(function() {

        deferred = $.Deferred();

        var model = new RequestModel({
          name: 'fake_csr',
          approvable: true,
          request_open: true
        });

        view.render();

        collection.add(model);

        child_view = view.children[model.cid];
      });

      function assert_controls_disabled() {
        expect(view.ui.accept_all.attr('disabled')).toBe('disabled');
        expect(view.ui.reject_all.attr('disabled')).toBe('disabled');
      }

      function assert_controls_enabled() {
        expect(view.ui.accept_all.attr('disabled')).toBeFalsy();
        expect(view.ui.reject_all.attr('disabled')).toBeFalsy();
      }

      _.each(['accept', 'reject'], function(action) {

        var method = action + '_all';

        describe(action + 'ing all', function() {

          beforeEach(function() {
            sinon.stub(collection, method).returns(deferred);
          });

          it('disables controls', function() {
            view[method]();

            assert_controls_disabled();
          });

          it('reenables controls on success', function() {
            view[method]();

            deferred.resolve();

            assert_controls_enabled();
          });

          it('reenables controls on failure', function() {
            view[method]();

            deferred.reject();

            assert_controls_enabled();
          });

          it('shows an error on failure', function() {
            view[method]();

            deferred.reject('Error');

            expect(view.ui.list_message[0].innerHTML).toMatch('Error');
          });

          it('hides any previously shown errors on invocation', function() {
            view[method]();

            deferred.reject('Error');

            // reset the deferred so it doesn't auto reject
            collection[method].restore();
            var deferred2 = $.Deferred();
            sinon.stub(collection, method).returns(deferred2);

            view[method]();

            expect(view.ui.list_message[0].innerHTML).toBeFalsy();
          });
        });

      });
    });

    describe('using user access control', function() {
      it('should not show action buttons to read-only users', function() {
        user.set('role', 'read-only');

        view.render();

        expect(view.ui.controls.length).toBe(0);
      });

      it('should show action buttons to read-write users', function() {
        user.set('role', 'read-write');

        view.render();

        expect(view.ui.controls.length).toBe(1);
      });

      it('show should action buttons to admin users', function() {
        user.set('role', 'admin');

        view.render();

        expect(view.ui.controls.length).toBe(1);
      });
    });
  });
});